@@ -1,6 +1,4 @@
-package org.apache.cassandra.cache;
 /*
- *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -9,79 +7,66 @@ package org.apache.cassandra.cache;
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing,
- * software distributed under the License is distributed on an
- * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- * KIND, either express or implied.  See the License for the
- * specific language governing permissions and limitations
- * under the License.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
+package org.apache.cassandra.cache;
 
-import java.util.Set;
+import java.util.Iterator;
 
 import com.googlecode.concurrentlinkedhashmap.ConcurrentLinkedHashMap;
-import com.googlecode.concurrentlinkedhashmap.Weigher;
-
-import com.googlecode.concurrentlinkedhashmap.Weighers;
+import com.googlecode.concurrentlinkedhashmap.EntryWeigher;
 
 /** Wrapper so CLHM can implement ICache interface.
  *  (this is what you get for making library classes final.) */
-public class ConcurrentLinkedHashCache<K, V> implements ICache<K, V>
+public class ConcurrentLinkedHashCache<K extends IMeasurableMemory, V extends IMeasurableMemory> implements ICache<K, V>
 {
     public static final int DEFAULT_CONCURENCY_LEVEL = 64;
     private final ConcurrentLinkedHashMap<K, V> map;
 
-    public ConcurrentLinkedHashCache(ConcurrentLinkedHashMap<K, V> map)
+    private ConcurrentLinkedHashCache(ConcurrentLinkedHashMap<K, V> map)
     {
         this.map = map;
     }
 
     /**
-     * Initialize a cache with weigher = Weighers.singleton() and initial capacity 0
-     *
-     * @param capacity cache weighted capacity
-     *
-     * @param <K> key type
-     * @param <V> value type
-     *
-     * @return initialized cache
+     * Initialize a cache with initial capacity with weightedCapacity
      */
-    public static <K, V> ConcurrentLinkedHashCache<K, V> create(int capacity)
-    {
-        return create(capacity, Weighers.<V>singleton());
-    }
-
-    /**
-     * Initialize a cache with initial capacity set to 0
-     *
-     * @param weightedCapacity cache weighted capacity
-     * @param weigher The weigher to use
-     *
-     * @param <K> key type
-     * @param <V> value type
-     *
-     * @return initialized cache
-     */
-    public static <K, V> ConcurrentLinkedHashCache<K, V> create(int weightedCapacity, Weigher<V> weigher)
+    public static <K extends IMeasurableMemory, V extends IMeasurableMemory> ConcurrentLinkedHashCache<K, V> create(long weightedCapacity, EntryWeigher<K, V> entryWeiger)
     {
         ConcurrentLinkedHashMap<K, V> map = new ConcurrentLinkedHashMap.Builder<K, V>()
-                                            .weigher(weigher)
+                                            .weigher(entryWeiger)
                                             .maximumWeightedCapacity(weightedCapacity)
                                             .concurrencyLevel(DEFAULT_CONCURENCY_LEVEL)
                                             .build();
 
-        return new ConcurrentLinkedHashCache<K, V>(map);
+        return new ConcurrentLinkedHashCache<>(map);
     }
 
-    public int capacity()
+    public static <K extends IMeasurableMemory, V extends IMeasurableMemory> ConcurrentLinkedHashCache<K, V> create(long weightedCapacity)
+    {
+        return create(weightedCapacity, new EntryWeigher<K, V>()
+        {
+            public int weightOf(K key, V value)
+            {
+                long size = key.unsharedHeapSize() + value.unsharedHeapSize();
+                assert size <= Integer.MAX_VALUE : "Serialized size cannot be more than 2GB/Integer.MAX_VALUE";
+                return (int) size;
+            }
+        });
+    }
+
+    public long capacity()
     {
         return map.capacity();
     }
 
-    public void setCapacity(int capacity)
+    public void setCapacity(long capacity)
     {
         map.setCapacity(capacity);
     }
@@ -96,7 +81,7 @@ public class ConcurrentLinkedHashCache<K, V> implements ICache<K, V>
         return map.size();
     }
 
-    public int weightedSize()
+    public long weightedSize()
     {
         return map.weightedSize();
     }
@@ -131,23 +116,18 @@ public class ConcurrentLinkedHashCache<K, V> implements ICache<K, V>
         map.remove(key);
     }
 
-    public Set<K> keySet()
+    public Iterator<K> keyIterator()
     {
-        return map.keySet();
+        return map.keySet().iterator();
     }
 
-    public Set<K> hotKeySet(int n)
+    public Iterator<K> hotKeyIterator(int n)
     {
-        return map.descendingKeySetWithLimit(n);
+        return map.descendingKeySetWithLimit(n).iterator();
     }
 
     public boolean containsKey(K key)
     {
         return map.containsKey(key);
     }
-
-    public boolean isPutCopying()
-    {
-        return false;
-    }
 }
