@@ -17,17 +17,25 @@
  */
 package org.apache.cassandra.config;
 
-import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 
 import com.google.common.base.Objects;
+import com.google.common.collect.ImmutableMap;
 
 public class ParameterizedClass
 {
+    public static final String CLASS_NAME = "class_name";
+    public static final String PARAMETERS = "parameters";
+
     public String class_name;
     public Map<String, String> parameters;
 
+    public ParameterizedClass()
+    {
+        // for snakeyaml
+    }
+
     public ParameterizedClass(String class_name, Map<String, String> parameters)
     {
         this.class_name = class_name;
@@ -35,10 +43,10 @@ public class ParameterizedClass
     }
 
     @SuppressWarnings("unchecked")
-    public ParameterizedClass(LinkedHashMap<String, ?> p)
+    public ParameterizedClass(Map<String, ?> p)
     {
-        this((String)p.get("class_name"),
-                p.containsKey("parameters") ? (Map<String, String>)((List<?>)p.get("parameters")).get(0) : null);
+        this((String)p.get(CLASS_NAME),
+             p.containsKey(PARAMETERS) ? (Map<String, String>)((List<?>)p.get(PARAMETERS)).get(0) : null);
     }
 
     @Override
