@@ -17,14 +17,12 @@
  */
 package org.apache.cassandra.cql3;
 
-import java.nio.ByteBuffer;
+import java.util.List;
 
-import org.apache.cassandra.config.CFMetaData;
 import org.apache.cassandra.config.ColumnDefinition;
-import org.apache.cassandra.db.ColumnFamily;
-import org.apache.cassandra.db.marshal.CollectionType;
-import org.apache.cassandra.db.marshal.CounterColumnType;
-import org.apache.cassandra.db.marshal.ListType;
+import org.apache.cassandra.cql3.functions.Function;
+import org.apache.cassandra.db.DecoratedKey;
+import org.apache.cassandra.db.marshal.*;
 import org.apache.cassandra.exceptions.InvalidRequestException;
 
 /**
@@ -43,34 +41,24 @@ import org.apache.cassandra.exceptions.InvalidRequestException;
  */
 public abstract class Operation
 {
-    // Name of the column the operation applies to
-    public final ColumnIdentifier columnName;
+    // the column the operation applies to
+    public final ColumnDefinition column;
 
     // Term involved in the operation. In theory this should not be here since some operation
     // may require none of more than one term, but most need 1 so it simplify things a bit.
     protected final Term t;
 
-    protected Operation(ColumnIdentifier columnName, Term t)
+    protected Operation(ColumnDefinition column, Term t)
     {
-        this.columnName = columnName;
+        assert column != null;
+        this.column = column;
         this.t = t;
     }
 
-    // Whether the colum operated on is a static column (on trunk, Operation stores the ColumnDefinition directly,
-    // not just the column name, so we'll be able to remove that lookup and check ColumnDefinition.isStatic field
-    // directly. But for 2.0, it's simpler that way).
-    public boolean isStatic(CFMetaData cfm)
+    public void addFunctionsTo(List<Function> functions)
     {
-        if (columnName == null)
-            return false;
-
-        ColumnDefinition def = cfm.getColumnDefinition(columnName.key);
-        return def != null && def.type == ColumnDefinition.Type.STATIC;
-    }
-
-    protected ColumnNameBuilder maybeUpdatePrefix(CFMetaData cfm, ColumnNameBuilder prefix)
-    {
-        return isStatic(cfm) ? cfm.getStaticColumnNameBuilder() : prefix;
+        if (t != null)
+            t.addFunctionsTo(functions);
     }
 
     /**
@@ -97,13 +85,10 @@ public abstract class Operation
     /**
      * Execute the operation.
      *
-     * @param rowKey row key for the update.
-     * @param cf the column family to which to add the updates generated by this operation.
-     * @param namePrefix the prefix that identify the CQL3 row this operation applies to (callers should not reuse
-     * the ColumnNameBuilder they pass here).
+     * @param partitionKey partition key for the update.
      * @param params parameters of the update.
      */
-    public abstract void execute(ByteBuffer rowKey, ColumnFamily cf, ColumnNameBuilder namePrefix, UpdateParameters params) throws InvalidRequestException;
+    public abstract void execute(DecoratedKey partitionKey, UpdateParameters params) throws InvalidRequestException;
 
     /**
      * A parsed raw UPDATE operation.
@@ -111,7 +96,7 @@ public abstract class Operation
      * This can be one of:
      *   - Setting a value: c = v
      *   - Setting an element of a collection: c[x] = v
-     *   - An addition/substraction to a variable: c = c +/- v (where v can be a collection literal)
+     *   - An addition/subtraction to a variable: c = c +/- v (where v can be a collection literal)
      *   - An prepend operation: c = v + c
      */
     public interface RawUpdate
@@ -128,7 +113,7 @@ public abstract class Operation
          * be a true column.
          * @return the prepared update operation.
          */
-        public Operation prepare(CFDefinition.Name receiver) throws InvalidRequestException;
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException;
 
         /**
          * @return whether this operation can be applied alongside the {@code
@@ -162,7 +147,7 @@ public abstract class Operation
          * @param receiver the "column" this operation applies to.
          * @return the prepared delete operation.
          */
-        public Operation prepare(ColumnSpecification receiver, CFMetaData cfm) throws InvalidRequestException;
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException;
     }
 
     public static class SetValue implements RawUpdate
@@ -174,24 +159,24 @@ public abstract class Operation
             this.value = value;
         }
 
-        public Operation prepare(CFDefinition.Name receiver) throws InvalidRequestException
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException
         {
-            Term v = value.prepare(receiver);
+            Term v = value.prepare(keyspace, receiver);
 
             if (receiver.type instanceof CounterColumnType)
-                throw new InvalidRequestException(String.format("Cannot set the value of counter column %s (counters can only be incremented/decremented, not set)", receiver));
+                throw new InvalidRequestException(String.format("Cannot set the value of counter column %s (counters can only be incremented/decremented, not set)", receiver.name));
 
-            if (!(receiver.type instanceof CollectionType))
-                return new Constants.Setter(receiver.kind == CFDefinition.Name.Kind.VALUE_ALIAS ? null : receiver.name, v);
+            if (!(receiver.type.isCollection()))
+                return new Constants.Setter(receiver, v);
 
             switch (((CollectionType)receiver.type).kind)
             {
                 case LIST:
-                    return new Lists.Setter(receiver.name, v);
+                    return new Lists.Setter(receiver, v);
                 case SET:
-                    return new Sets.Setter(receiver.name, v);
+                    return new Sets.Setter(receiver, v);
                 case MAP:
-                    return new Maps.Setter(receiver.name, v);
+                    return new Maps.Setter(receiver, v);
             }
             throw new AssertionError();
         }
@@ -207,6 +192,11 @@ public abstract class Operation
             // it's stupid and 2) the result would seem random to the user.
             return false;
         }
+
+        public Term.Raw value()
+        {
+            return value;
+        }
     }
 
     public static class SetElement implements RawUpdate
@@ -220,30 +210,32 @@ public abstract class Operation
             this.value = value;
         }
 
-        public Operation prepare(CFDefinition.Name receiver) throws InvalidRequestException
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException
         {
             if (!(receiver.type instanceof CollectionType))
-                throw new InvalidRequestException(String.format("Invalid operation (%s) for non collection column %s", toString(receiver), receiver));
+                throw new InvalidRequestException(String.format("Invalid operation (%s) for non collection column %s", toString(receiver), receiver.name));
+            else if (!(receiver.type.isMultiCell()))
+                throw new InvalidRequestException(String.format("Invalid operation (%s) for frozen collection column %s", toString(receiver), receiver.name));
 
             switch (((CollectionType)receiver.type).kind)
             {
                 case LIST:
-                    Term idx = selector.prepare(Lists.indexSpecOf(receiver));
-                    Term lval = value.prepare(Lists.valueSpecOf(receiver));
-                    return new Lists.SetterByIndex(receiver.name, idx, lval);
+                    Term idx = selector.prepare(keyspace, Lists.indexSpecOf(receiver));
+                    Term lval = value.prepare(keyspace, Lists.valueSpecOf(receiver));
+                    return new Lists.SetterByIndex(receiver, idx, lval);
                 case SET:
-                    throw new InvalidRequestException(String.format("Invalid operation (%s) for set column %s", toString(receiver), receiver));
+                    throw new InvalidRequestException(String.format("Invalid operation (%s) for set column %s", toString(receiver), receiver.name));
                 case MAP:
-                    Term key = selector.prepare(Maps.keySpecOf(receiver));
-                    Term mval = value.prepare(Maps.valueSpecOf(receiver));
-                    return new Maps.SetterByKey(receiver.name, key, mval);
+                    Term key = selector.prepare(keyspace, Maps.keySpecOf(receiver));
+                    Term mval = value.prepare(keyspace, Maps.valueSpecOf(receiver));
+                    return new Maps.SetterByKey(receiver, key, mval);
             }
             throw new AssertionError();
         }
 
         protected String toString(ColumnSpecification column)
         {
-            return String.format("%s[%s] = %s", column, selector, value);
+            return String.format("%s[%s] = %s", column.name, selector, value);
         }
 
         public boolean isCompatibleWith(RawUpdate other)
@@ -254,6 +246,72 @@ public abstract class Operation
         }
     }
 
+    // Currently only used internally counters support in SuperColumn families.
+    // Addition on the element level inside the collections are otherwise not supported in the CQL.
+    public static class ElementAddition implements RawUpdate
+    {
+        private final Term.Raw selector;
+        private final Term.Raw value;
+
+        public ElementAddition(Term.Raw selector, Term.Raw value)
+        {
+            this.selector = selector;
+            this.value = value;
+        }
+
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException
+        {
+            assert receiver.type instanceof MapType;
+            Term k = selector.prepare(keyspace, Maps.keySpecOf(receiver));
+            Term v = value.prepare(keyspace, Maps.valueSpecOf(receiver));
+
+            return new Maps.AdderByKey(receiver, v, k);
+        }
+
+        protected String toString(ColumnSpecification column)
+        {
+            return String.format("%s = %s + %s", column.name, column.name, value);
+        }
+
+        public boolean isCompatibleWith(RawUpdate other)
+        {
+            return !(other instanceof SetValue);
+        }
+    }
+
+    // Currently only used internally counters support in SuperColumn families.
+    // Addition on the element level inside the collections are otherwise not supported in the CQL.
+    public static class ElementSubtraction implements RawUpdate
+    {
+        private final Term.Raw selector;
+        private final Term.Raw value;
+
+        public  ElementSubtraction(Term.Raw selector, Term.Raw value)
+        {
+            this.selector = selector;
+            this.value = value;
+        }
+
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException
+        {
+            assert receiver.type instanceof MapType;
+            Term k = selector.prepare(keyspace, Maps.keySpecOf(receiver));
+            Term v = value.prepare(keyspace, Maps.valueSpecOf(receiver));
+
+            return new Maps.SubtracterByKey(receiver, v, k);
+        }
+
+        protected String toString(ColumnSpecification column)
+        {
+            return String.format("%s = %s + %s", column.name, column.name, value);
+        }
+
+        public boolean isCompatibleWith(RawUpdate other)
+        {
+            return !(other instanceof SetValue);
+        }
+    }
+
     public static class Addition implements RawUpdate
     {
         private final Term.Raw value;
@@ -263,38 +321,45 @@ public abstract class Operation
             this.value = value;
         }
 
-        public Operation prepare(CFDefinition.Name receiver) throws InvalidRequestException
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException
         {
-            Term v = value.prepare(receiver);
+            Term v = value.prepare(keyspace, receiver);
 
             if (!(receiver.type instanceof CollectionType))
             {
                 if (!(receiver.type instanceof CounterColumnType))
-                    throw new InvalidRequestException(String.format("Invalid operation (%s) for non counter column %s", toString(receiver), receiver));
-                return new Constants.Adder(receiver.kind == CFDefinition.Name.Kind.VALUE_ALIAS ? null : receiver.name, v);
+                    throw new InvalidRequestException(String.format("Invalid operation (%s) for non counter column %s", toString(receiver), receiver.name));
+                return new Constants.Adder(receiver, v);
             }
+            else if (!(receiver.type.isMultiCell()))
+                throw new InvalidRequestException(String.format("Invalid operation (%s) for frozen collection column %s", toString(receiver), receiver.name));
 
             switch (((CollectionType)receiver.type).kind)
             {
                 case LIST:
-                    return new Lists.Appender(receiver.name, v);
+                    return new Lists.Appender(receiver, v);
                 case SET:
-                    return new Sets.Adder(receiver.name, v);
+                    return new Sets.Adder(receiver, v);
                 case MAP:
-                    return new Maps.Putter(receiver.name, v);
+                    return new Maps.Putter(receiver, v);
             }
             throw new AssertionError();
         }
 
         protected String toString(ColumnSpecification column)
         {
-            return String.format("%s = %s + %s", column, column, value);
+            return String.format("%s = %s + %s", column.name, column.name, value);
         }
 
         public boolean isCompatibleWith(RawUpdate other)
         {
             return !(other instanceof SetValue);
         }
+
+        public Term.Raw value()
+        {
+            return value;
+        }
     }
 
     public static class Substraction implements RawUpdate
@@ -306,38 +371,48 @@ public abstract class Operation
             this.value = value;
         }
 
-        public Operation prepare(CFDefinition.Name receiver) throws InvalidRequestException
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException
         {
-            Term v = value.prepare(receiver);
-
             if (!(receiver.type instanceof CollectionType))
             {
                 if (!(receiver.type instanceof CounterColumnType))
-                    throw new InvalidRequestException(String.format("Invalid operation (%s) for non counter column %s", toString(receiver), receiver));
-                return new Constants.Substracter(receiver.kind == CFDefinition.Name.Kind.VALUE_ALIAS ? null : receiver.name, v);
+                    throw new InvalidRequestException(String.format("Invalid operation (%s) for non counter column %s", toString(receiver), receiver.name));
+                return new Constants.Substracter(receiver, value.prepare(keyspace, receiver));
             }
+            else if (!(receiver.type.isMultiCell()))
+                throw new InvalidRequestException(String.format("Invalid operation (%s) for frozen collection column %s", toString(receiver), receiver.name));
 
             switch (((CollectionType)receiver.type).kind)
             {
                 case LIST:
-                    return new Lists.Discarder(receiver.name, v);
+                    return new Lists.Discarder(receiver, value.prepare(keyspace, receiver));
                 case SET:
-                    return new Sets.Discarder(receiver.name, v);
+                    return new Sets.Discarder(receiver, value.prepare(keyspace, receiver));
                 case MAP:
-                    throw new InvalidRequestException(String.format("Invalid operation (%s) for map column %s", toString(receiver), receiver));
+                    // The value for a map subtraction is actually a set
+                    ColumnSpecification vr = new ColumnSpecification(receiver.ksName,
+                                                                     receiver.cfName,
+                                                                     receiver.name,
+                                                                     SetType.getInstance(((MapType)receiver.type).getKeysType(), false));
+                    return new Sets.Discarder(receiver, value.prepare(keyspace, vr));
             }
             throw new AssertionError();
         }
 
         protected String toString(ColumnSpecification column)
         {
-            return String.format("%s = %s - %s", column, column, value);
+            return String.format("%s = %s - %s", column.name, column.name, value);
         }
 
         public boolean isCompatibleWith(RawUpdate other)
         {
             return !(other instanceof SetValue);
         }
+
+        public Term.Raw value()
+        {
+            return value;
+        }
     }
 
     public static class Prepend implements RawUpdate
@@ -349,19 +424,21 @@ public abstract class Operation
             this.value = value;
         }
 
-        public Operation prepare(CFDefinition.Name receiver) throws InvalidRequestException
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException
         {
-            Term v = value.prepare(receiver);
+            Term v = value.prepare(keyspace, receiver);
 
             if (!(receiver.type instanceof ListType))
-                throw new InvalidRequestException(String.format("Invalid operation (%s) for non list column %s", toString(receiver), receiver));
+                throw new InvalidRequestException(String.format("Invalid operation (%s) for non list column %s", toString(receiver), receiver.name));
+            else if (!(receiver.type.isMultiCell()))
+                throw new InvalidRequestException(String.format("Invalid operation (%s) for frozen list column %s", toString(receiver), receiver.name));
 
-            return new Lists.Prepender(receiver.name, v);
+            return new Lists.Prepender(receiver, v);
         }
 
         protected String toString(ColumnSpecification column)
         {
-            return String.format("%s = %s - %s", column, value, column);
+            return String.format("%s = %s - %s", column.name, value, column.name);
         }
 
         public boolean isCompatibleWith(RawUpdate other)
@@ -384,10 +461,10 @@ public abstract class Operation
             return id;
         }
 
-        public Operation prepare(ColumnSpecification receiver, CFMetaData cfm) throws InvalidRequestException
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException
         {
             // No validation, deleting a column is always "well typed"
-            return new Constants.Deleter(id.prepare(cfm), receiver.type instanceof CollectionType);
+            return new Constants.Deleter(receiver);
         }
     }
 
@@ -407,23 +484,24 @@ public abstract class Operation
             return id;
         }
 
-        public Operation prepare(ColumnSpecification receiver, CFMetaData cfm) throws InvalidRequestException
+        public Operation prepare(String keyspace, ColumnDefinition receiver) throws InvalidRequestException
         {
-            if (!(receiver.type instanceof CollectionType))
-                throw new InvalidRequestException(String.format("Invalid deletion operation for non collection column %s", receiver));
+            if (!(receiver.type.isCollection()))
+                throw new InvalidRequestException(String.format("Invalid deletion operation for non collection column %s", receiver.name));
+            else if (!(receiver.type.isMultiCell()))
+                throw new InvalidRequestException(String.format("Invalid deletion operation for frozen collection column %s", receiver.name));
 
-            ColumnIdentifier preparedId = id.prepare(cfm);
             switch (((CollectionType)receiver.type).kind)
             {
                 case LIST:
-                    Term idx = element.prepare(Lists.indexSpecOf(receiver));
-                    return new Lists.DiscarderByIndex(preparedId, idx);
+                    Term idx = element.prepare(keyspace, Lists.indexSpecOf(receiver));
+                    return new Lists.DiscarderByIndex(receiver, idx);
                 case SET:
-                    Term elt = element.prepare(Sets.valueSpecOf(receiver));
-                    return new Sets.Discarder(preparedId, elt);
+                    Term elt = element.prepare(keyspace, Sets.valueSpecOf(receiver));
+                    return new Sets.ElementDiscarder(receiver, elt);
                 case MAP:
-                    Term key = element.prepare(Maps.keySpecOf(receiver));
-                    return new Maps.DiscarderByKey(preparedId, key);
+                    Term key = element.prepare(keyspace, Maps.keySpecOf(receiver));
+                    return new Maps.DiscarderByKey(receiver, key);
             }
             throw new AssertionError();
         }
