@@ -18,25 +18,23 @@
 package org.apache.cassandra.cql3.functions;
 
 import java.nio.ByteBuffer;
-import java.util.List;
-import java.util.UUID;
+import java.util.*;
 
 import org.apache.cassandra.db.marshal.UUIDType;
 import org.apache.cassandra.serializers.UUIDSerializer;
 
 public abstract class UuidFcts
 {
-    public static final Function uuidFct = new AbstractFunction("uuid", UUIDType.instance)
+    public static Collection<Function> all()
     {
-        public ByteBuffer execute(List<ByteBuffer> parameters)
+        return Collections.singleton(uuidFct);
+    }
+
+    public static final Function uuidFct = new NativeScalarFunction("uuid", UUIDType.instance)
+    {
+        public ByteBuffer execute(int protocolVersion, List<ByteBuffer> parameters)
         {
             return UUIDSerializer.instance.serialize(UUID.randomUUID());
         }
-
-        @Override
-        public boolean isPure()
-        {
-            return false;
-        }
     };
 }
