@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -15,19 +15,20 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.cassandra.db;
 
 import java.nio.ByteBuffer;
-import java.io.IOException;
 import java.util.Collection;
+import java.util.UUID;
+
+import org.apache.cassandra.db.partitions.PartitionUpdate;
 
 public interface IMutation
 {
-    public String getTable();
-    public Collection<Integer> getColumnFamilyIds();
-    public ByteBuffer key();
-    public void apply() throws IOException;
+    public String getKeyspaceName();
+    public Collection<UUID> getColumnFamilyIds();
+    public DecoratedKey key();
+    public long getTimeout();
     public String toString(boolean shallow);
-    public void addAll(IMutation m);
+    public Collection<PartitionUpdate> getPartitionUpdates();
 }
