@@ -552,7 +552,8 @@ public final class SystemKeyspace
             SystemKeyspace.getOrInitializeLocalHostId(nodeIdSupplier);
     }
 
-    public static void updateCompactionHistory(String ksname,
+    public static void updateCompactionHistory(UUID taskId,
+                                               String ksname,
                                                String cfname,
                                                long compactedAt,
                                                long bytesIn,
@@ -564,7 +565,7 @@ public final class SystemKeyspace
             return;
         String req = "INSERT INTO system.%s (id, keyspace_name, columnfamily_name, compacted_at, bytes_in, bytes_out, rows_merged) VALUES (?, ?, ?, ?, ?, ?, ?)";
         executeInternal(String.format(req, COMPACTION_HISTORY),
-                        UUIDGen.getTimeUUID(),
+                        taskId,
                         ksname,
                         cfname,
                         ByteBufferUtil.bytes(compactedAt),
