@@ -67,6 +67,9 @@ final class LogFile implements AutoCloseable
 
     // The transaction records, this set must be ORDER PRESERVING
     private final LinkedHashSet<LogRecord> records = new LinkedHashSet<>();
+    // the transaction records we have written to disk - used to guarantee that the
+    // on-disk log files become identical when creating a new replica
+    private final LinkedHashSet<LogRecord> onDiskRecords = new LinkedHashSet<>();
 
     // The type of the transaction
     private final OperationType type;
@@ -320,17 +323,13 @@ final class LogFile implements AutoCloseable
         assert type == Type.ADD || type == Type.REMOVE;
 
         for (SSTableReader sstable : tables)
-        {
-            File folder = sstable.descriptor.directory;
-            replicas.maybeCreateReplica(folder, getFileName(folder), records);
-        }
+            maybeCreateReplica(sstable);
         return LogRecord.make(type, tables);
     }
 
     private LogRecord makeAddRecord(SSTable table)
     {
-        File folder = table.descriptor.directory;
-        replicas.maybeCreateReplica(folder, getFileName(folder), records);
+        maybeCreateReplica(table);
         return LogRecord.make(Type.ADD, table);
     }
 
@@ -342,12 +341,17 @@ final class LogFile implements AutoCloseable
     private LogRecord makeRecord(Type type, SSTable table, LogRecord record)
     {
         assert type == Type.ADD || type == Type.REMOVE;
-
-        File folder = table.descriptor.directory;
-        replicas.maybeCreateReplica(folder, getFileName(folder), records);
+        maybeCreateReplica(table);
         return record.asType(type);
     }
 
+    private void maybeCreateReplica(SSTable sstable)
+    {
+        File folder = sstable.descriptor.directory;
+        String fileName = getFileName(folder);
+        replicas.maybeCreateReplica(folder, fileName, onDiskRecords);
+    }
+
     void addRecord(LogRecord record)
     {
         if (completed())
@@ -359,6 +363,7 @@ final class LogFile implements AutoCloseable
         replicas.append(record);
         if (!records.add(record))
             throw new IllegalStateException("Failed to add record");
+        onDiskRecords.add(record);
     }
 
     void remove(SSTable table)
