@@ -31,9 +31,9 @@ import org.apache.cassandra.net.MessagingService;
 
 public class GossipDigestSynVerbHandler implements IVerbHandler<GossipDigestSyn>
 {
-    private static final Logger logger = LoggerFactory.getLogger( GossipDigestSynVerbHandler.class);
+    private static final Logger logger = LoggerFactory.getLogger(GossipDigestSynVerbHandler.class);
 
-    public void doVerb(MessageIn<GossipDigestSyn> message, String id)
+    public void doVerb(MessageIn<GossipDigestSyn> message, int id)
     {
         InetAddress from = message.from;
         if (logger.isTraceEnabled())
@@ -49,13 +49,13 @@ public class GossipDigestSynVerbHandler implements IVerbHandler<GossipDigestSyn>
         /* If the message is from a different cluster throw it away. */
         if (!gDigestMessage.clusterId.equals(DatabaseDescriptor.getClusterName()))
         {
-            logger.warn("ClusterName mismatch from " + from + " " + gDigestMessage.clusterId  + "!=" + DatabaseDescriptor.getClusterName());
+            logger.warn("ClusterName mismatch from {} {}!={}", from, gDigestMessage.clusterId, DatabaseDescriptor.getClusterName());
             return;
         }
 
         if (gDigestMessage.partioner != null && !gDigestMessage.partioner.equals(DatabaseDescriptor.getPartitionerName()))
         {
-            logger.warn("Partitioner mismatch from " + from + " " + gDigestMessage.partioner  + "!=" + DatabaseDescriptor.getPartitionerName());
+            logger.warn("Partitioner mismatch from {} {}!={}", from, gDigestMessage.partioner, DatabaseDescriptor.getPartitionerName());
             return;
         }
 
@@ -63,12 +63,12 @@ public class GossipDigestSynVerbHandler implements IVerbHandler<GossipDigestSyn>
         if (logger.isTraceEnabled())
         {
             StringBuilder sb = new StringBuilder();
-            for ( GossipDigest gDigest : gDigestList )
+            for (GossipDigest gDigest : gDigestList)
             {
                 sb.append(gDigest);
                 sb.append(" ");
             }
-            logger.trace("Gossip syn digests are : " + sb.toString());
+            logger.trace("Gossip syn digests are : {}", sb);
         }
 
         doSort(gDigestList);
@@ -76,10 +76,10 @@ public class GossipDigestSynVerbHandler implements IVerbHandler<GossipDigestSyn>
         List<GossipDigest> deltaGossipDigestList = new ArrayList<GossipDigest>();
         Map<InetAddress, EndpointState> deltaEpStateMap = new HashMap<InetAddress, EndpointState>();
         Gossiper.instance.examineGossiper(gDigestList, deltaGossipDigestList, deltaEpStateMap);
-
+        logger.trace("sending {} digests and {} deltas", deltaGossipDigestList.size(), deltaEpStateMap.size());
         MessageOut<GossipDigestAck> gDigestAckMessage = new MessageOut<GossipDigestAck>(MessagingService.Verb.GOSSIP_DIGEST_ACK,
-                                                                                                      new GossipDigestAck(deltaGossipDigestList, deltaEpStateMap),
-                                                                                                      GossipDigestAck.serializer);
+                                                                                        new GossipDigestAck(deltaGossipDigestList, deltaEpStateMap),
+                                                                                        GossipDigestAck.serializer);
         if (logger.isTraceEnabled())
             logger.trace("Sending a GossipDigestAckMessage to {}", from);
         MessagingService.instance().sendOneWay(gDigestAckMessage, from);
@@ -96,7 +96,7 @@ public class GossipDigestSynVerbHandler implements IVerbHandler<GossipDigestSyn>
     {
         /* Construct a map of endpoint to GossipDigest. */
         Map<InetAddress, GossipDigest> epToDigestMap = new HashMap<InetAddress, GossipDigest>();
-        for ( GossipDigest gDigest : gDigestList )
+        for (GossipDigest gDigest : gDigestList)
         {
             epToDigestMap.put(gDigest.getEndpoint(), gDigest);
         }
@@ -106,13 +106,13 @@ public class GossipDigestSynVerbHandler implements IVerbHandler<GossipDigestSyn>
          * of the local EndpointState and the version found in the GossipDigest.
         */
         List<GossipDigest> diffDigests = new ArrayList<GossipDigest>(gDigestList.size());
-        for ( GossipDigest gDigest : gDigestList )
+        for (GossipDigest gDigest : gDigestList)
         {
             InetAddress ep = gDigest.getEndpoint();
             EndpointState epState = Gossiper.instance.getEndpointStateForEndpoint(ep);
-            int version = (epState != null) ? Gossiper.instance.getMaxEndpointStateVersion( epState ) : 0;
-            int diffVersion = Math.abs(version - gDigest.getMaxVersion() );
-            diffDigests.add( new GossipDigest(ep, gDigest.getGeneration(), diffVersion) );
+            int version = (epState != null) ? Gossiper.instance.getMaxEndpointStateVersion(epState) : 0;
+            int diffVersion = Math.abs(version - gDigest.getMaxVersion());
+            diffDigests.add(new GossipDigest(ep, gDigest.getGeneration(), diffVersion));
         }
 
         gDigestList.clear();
@@ -122,9 +122,9 @@ public class GossipDigestSynVerbHandler implements IVerbHandler<GossipDigestSyn>
          * Report the digests in descending order. This takes care of the endpoints
          * that are far behind w.r.t this local endpoint
         */
-        for ( int i = size - 1; i >= 0; --i )
+        for (int i = size - 1; i >= 0; --i)
         {
-            gDigestList.add( epToDigestMap.get(diffDigests.get(i).getEndpoint()) );
+            gDigestList.add(epToDigestMap.get(diffDigests.get(i).getEndpoint()));
         }
     }
 }
