@@ -19,39 +19,84 @@ package org.apache.cassandra.io.util;
 
 import com.google.common.util.concurrent.RateLimiter;
 
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import org.apache.cassandra.config.Config;
+import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.io.compress.CompressedRandomAccessReader;
 import org.apache.cassandra.io.compress.CompressedSequentialWriter;
-import org.apache.cassandra.io.compress.CompressedThrottledReader;
 import org.apache.cassandra.io.compress.CompressionMetadata;
-import org.apache.cassandra.io.sstable.SSTableWriter;
+import org.apache.cassandra.utils.JVMStabilityInspector;
+import org.apache.cassandra.utils.concurrent.Ref;
 
 public class CompressedSegmentedFile extends SegmentedFile implements ICompressedFile
 {
-    public final CompressionMetadata metadata;
+    private static final Logger logger = LoggerFactory.getLogger(CompressedSegmentedFile.class);
+    private static final boolean useMmap = DatabaseDescriptor.getDiskAccessMode() == Config.DiskAccessMode.mmap;
 
-    public CompressedSegmentedFile(String path, CompressionMetadata metadata)
+    public final CompressionMetadata metadata;
+    private final MmappedRegions regions;
+
+    public CompressedSegmentedFile(ChannelProxy channel, int bufferSize, CompressionMetadata metadata)
     {
-        super(new Cleanup(path, metadata), path, metadata.dataLength, metadata.compressedFileLength);
+        this(channel,
+             bufferSize,
+             metadata,
+             useMmap
+             ? MmappedRegions.map(channel, metadata)
+             : null);
+    }
+
+    public CompressedSegmentedFile(ChannelProxy channel, int bufferSize, CompressionMetadata metadata, MmappedRegions regions)
+    {
+        super(new Cleanup(channel, metadata, regions), channel, bufferSize, metadata.dataLength, metadata.compressedFileLength);
         this.metadata = metadata;
+        this.regions = regions;
     }
 
     private CompressedSegmentedFile(CompressedSegmentedFile copy)
     {
         super(copy);
         this.metadata = copy.metadata;
+        this.regions = copy.regions;
+    }
+
+    public ChannelProxy channel()
+    {
+        return channel;
+    }
+
+    public MmappedRegions regions()
+    {
+        return regions;
     }
 
     private static final class Cleanup extends SegmentedFile.Cleanup
     {
         final CompressionMetadata metadata;
-        protected Cleanup(String path, CompressionMetadata metadata)
+        private final MmappedRegions regions;
+
+        protected Cleanup(ChannelProxy channel, CompressionMetadata metadata, MmappedRegions regions)
         {
-            super(path);
+            super(channel);
             this.metadata = metadata;
+            this.regions = regions;
         }
-        public void tidy() throws Exception
+        public void tidy()
         {
+            Throwable err = regions == null ? null : regions.close(null);
+            if (err != null)
+            {
+                JVMStabilityInspector.inspectThrowable(err);
+
+                // This is not supposed to happen
+                logger.error("Error while closing mmapped regions", err);
+            }
+
             metadata.close();
+
+            super.tidy();
         }
     }
 
@@ -60,31 +105,31 @@ public class CompressedSegmentedFile extends SegmentedFile implements ICompresse
         return new CompressedSegmentedFile(this);
     }
 
+    public void addTo(Ref.IdentityCollection identities)
+    {
+        super.addTo(identities);
+        metadata.addTo(identities);
+    }
+
     public static class Builder extends SegmentedFile.Builder
     {
-        protected final CompressedSequentialWriter writer;
+        final CompressedSequentialWriter writer;
         public Builder(CompressedSequentialWriter writer)
         {
             this.writer = writer;
         }
 
-        public void addPotentialBoundary(long boundary)
-        {
-            // only one segment in a standard-io file
-        }
-
-        protected CompressionMetadata metadata(String path, long overrideLength, boolean isFinal)
+        protected CompressionMetadata metadata(String path, long overrideLength)
         {
             if (writer == null)
                 return CompressionMetadata.create(path);
 
-            return writer.open(overrideLength, isFinal);
+            return writer.open(overrideLength);
         }
 
-        public SegmentedFile complete(String path, long overrideLength, boolean isFinal)
+        public SegmentedFile complete(ChannelProxy channel, int bufferSize, long overrideLength)
         {
-            assert !isFinal || overrideLength <= 0;
-            return new CompressedSegmentedFile(path, metadata(path, overrideLength, isFinal));
+            return new CompressedSegmentedFile(channel, bufferSize, metadata(channel.filePath(), overrideLength));
         }
     }
 
@@ -97,12 +142,12 @@ public class CompressedSegmentedFile extends SegmentedFile implements ICompresse
 
     public RandomAccessReader createReader()
     {
-        return CompressedRandomAccessReader.open(path, metadata);
+        return new CompressedRandomAccessReader.Builder(this).build();
     }
 
-    public RandomAccessReader createThrottledReader(RateLimiter limiter)
+    public RandomAccessReader createReader(RateLimiter limiter)
     {
-        return CompressedThrottledReader.open(path, metadata, limiter);
+        return new CompressedRandomAccessReader.Builder(this).limiter(limiter).build();
     }
 
     public CompressionMetadata getMetadata()
