@@ -37,7 +37,7 @@ public class DataOutputBuffer extends BufferedDataOutputStreamPlus
     /*
      * Threshold at which resizing transitions from doubling to increasing by 50%
      */
-    private static final long DOUBLING_THRESHOLD = Long.getLong(Config.PROPERTY_PREFIX + "DOB_DOUBLING_THRESHOLD_MB", 64);
+    static final long DOUBLING_THRESHOLD = Long.getLong(Config.PROPERTY_PREFIX + "DOB_DOUBLING_THRESHOLD_MB", 64);
 
     public DataOutputBuffer()
     {
@@ -83,7 +83,7 @@ public class DataOutputBuffer extends BufferedDataOutputStreamPlus
     @Override
     protected void doFlush(int count) throws IOException
     {
-        reallocate(count);
+        expandToFit(count);
     }
 
     //Hack for test, make it possible to override checking the buffer capacity
@@ -119,7 +119,7 @@ public class DataOutputBuffer extends BufferedDataOutputStreamPlus
         return validateReallocation(newSize);
     }
 
-    protected void reallocate(long count)
+    protected void expandToFit(long count)
     {
         if (count <= 0)
             return;
@@ -141,7 +141,7 @@ public class DataOutputBuffer extends BufferedDataOutputStreamPlus
         public int write(ByteBuffer src) throws IOException
         {
             int count = src.remaining();
-            reallocate(count);
+            expandToFit(count);
             buffer.put(src);
             return count;
         }
@@ -151,7 +151,7 @@ public class DataOutputBuffer extends BufferedDataOutputStreamPlus
             return true;
         }
 
-        public void close() throws IOException
+        public void close()
         {
         }
     }
@@ -163,6 +163,19 @@ public class DataOutputBuffer extends BufferedDataOutputStreamPlus
 
     public ByteBuffer buffer()
     {
+        return buffer(true);
+    }
+
+    public ByteBuffer buffer(boolean duplicate)
+    {
+        if (!duplicate)
+        {
+            ByteBuffer buf = buffer;
+            buf.flip();
+            buffer = null;
+            return buf;
+        }
+
         ByteBuffer result = buffer.duplicate();
         result.flip();
         return result;
@@ -178,6 +191,16 @@ public class DataOutputBuffer extends BufferedDataOutputStreamPlus
         return buffer.position();
     }
 
+    public boolean hasPosition()
+    {
+        return true;
+    }
+
+    public long position()
+    {
+        return getLength();
+    }
+
     public byte[] toByteArray()
     {
         ByteBuffer buffer = buffer();
