@@ -17,13 +17,15 @@
  */
 package org.apache.cassandra.metrics;
 
-import java.lang.management.ManagementFactory;
 import java.lang.reflect.Method;
 import java.util.Locale;
 import java.util.concurrent.TimeUnit;
 
 import com.codahale.metrics.*;
-import javax.management.*;
+import javax.management.MalformedObjectNameException;
+import javax.management.ObjectName;
+
+import org.apache.cassandra.utils.MBeanWrapper;
 
 /**
  * Makes integrating 3.0 metrics API with 2.0.
@@ -35,7 +37,7 @@ public class CassandraMetricsRegistry extends MetricRegistry
 {
     public static final CassandraMetricsRegistry Metrics = new CassandraMetricsRegistry();
 
-    private final MBeanServer mBeanServer = ManagementFactory.getPlatformMBeanServer();
+    private final MBeanWrapper mBeanServer = MBeanWrapper.instance;
 
     private CassandraMetricsRegistry()
     {
@@ -50,6 +52,13 @@ public class CassandraMetricsRegistry extends MetricRegistry
         return counter;
     }
 
+    public Counter counter(MetricName name, MetricName alias)
+    {
+        Counter counter = counter(name);
+        registerAlias(name, alias);
+        return counter;
+    }
+
     public Meter meter(MetricName name)
     {
         Meter meter = meter(name.getMetricName());
@@ -58,22 +67,43 @@ public class CassandraMetricsRegistry extends MetricRegistry
         return meter;
     }
 
+    public Meter meter(MetricName name, MetricName alias)
+    {
+        Meter meter = meter(name);
+        registerAlias(name, alias);
+        return meter;
+    }
+
     public Histogram histogram(MetricName name, boolean considerZeroes)
     {
-        Histogram histogram = register(name, new ClearableHistogram(new EstimatedHistogramReservoir(considerZeroes)));
+        Histogram histogram = register(name, new ClearableHistogram(new DecayingEstimatedHistogramReservoir(considerZeroes)));
         registerMBean(histogram, name.getMBeanName());
 
         return histogram;
     }
 
+    public Histogram histogram(MetricName name, MetricName alias, boolean considerZeroes)
+    {
+        Histogram histogram = histogram(name, considerZeroes);
+        registerAlias(name, alias);
+        return histogram;
+    }
+
     public Timer timer(MetricName name)
     {
-        Timer timer = register(name, new Timer(new EstimatedHistogramReservoir(false)));
+        Timer timer = register(name, new Timer(new DecayingEstimatedHistogramReservoir()));
         registerMBean(timer, name.getMBeanName());
 
         return timer;
     }
 
+    public Timer timer(MetricName name, MetricName alias)
+    {
+        Timer timer = timer(name);
+        registerAlias(name, alias);
+        return timer;
+    }
+
     public <T extends Metric> T register(MetricName name, T metric)
     {
         try
@@ -89,6 +119,13 @@ public class CassandraMetricsRegistry extends MetricRegistry
         }
     }
 
+    public <T extends Metric> T register(MetricName name, MetricName aliasName, T metric)
+    {
+        T ret = register(name, metric);
+        registerAlias(name, aliasName);
+        return ret;
+    }
+
     public boolean remove(MetricName name)
     {
         boolean removed = remove(name.getMetricName());
@@ -101,6 +138,16 @@ public class CassandraMetricsRegistry extends MetricRegistry
         return removed;
     }
 
+    public boolean remove(MetricName name, MetricName alias)
+    {
+        if (remove(name))
+        {
+            removeAlias(alias);
+            return true;
+        }
+        return false;
+    }
+
     public void registerMBean(Metric metric, ObjectName name)
     {
         AbstractBean mbean;
@@ -128,6 +175,23 @@ public class CassandraMetricsRegistry extends MetricRegistry
         try
         {
             mBeanServer.registerMBean(mbean, name);
+        }
+        catch (Exception ignored) {}
+    }
+
+    private void registerAlias(MetricName existingName, MetricName aliasName)
+    {
+        Metric existing = Metrics.getMetrics().get(existingName.getMetricName());
+        assert existing != null : existingName + " not registered";
+
+        registerMBean(existing, aliasName.getMBeanName());
+    }
+
+    private void removeAlias(MetricName name)
+    {
+        try
+        {
+            MBeanWrapper.instance.unregisterMBean(name.getMBeanName());
         } catch (Exception ignored) {}
     }
 
