@@ -17,16 +17,36 @@
  */
 package org.apache.cassandra.net;
 
+import java.net.InetAddress;
+
+import com.google.common.base.Predicate;
+
+import org.apache.cassandra.gms.FailureDetector;
+
 /**
  * implementors of IAsyncCallback need to make sure that any public methods
  * are threadsafe with respect to response() being called from the message
  * service.  In particular, if any shared state is referenced, making
  * response alone synchronized will not suffice.
  */
-public interface IAsyncCallback<T> extends IMessageCallback
+public interface IAsyncCallback<T>
 {
+    public static Predicate<InetAddress> isAlive = new Predicate<InetAddress>()
+    {
+        public boolean apply(InetAddress endpoint)
+        {
+            return FailureDetector.instance.isAlive(endpoint);
+        }
+    };
+
     /**
      * @param msg response received.
      */
     public void response(MessageIn<T> msg);
+
+    /**
+     * @return true if this callback is on the read path and its latency should be
+     * given as input to the dynamic snitch.
+     */
+    boolean isLatencyForSnitch();
 }
