@@ -28,7 +28,7 @@ import org.apache.cassandra.db.WriteType;
 /**
  * This class blocks for a quorum of responses _in the local datacenter only_ (CL.LOCAL_QUORUM).
  */
-public class DatacenterWriteResponseHandler extends WriteResponseHandler
+public class DatacenterWriteResponseHandler<T> extends WriteResponseHandler<T>
 {
     public DatacenterWriteResponseHandler(Collection<InetAddress> naturalEndpoints,
                                           Collection<InetAddress> pendingEndpoints,
@@ -42,9 +42,9 @@ public class DatacenterWriteResponseHandler extends WriteResponseHandler
     }
 
     @Override
-    public void response(MessageIn message)
+    public void response(MessageIn<T> message)
     {
-        if (message == null || consistencyLevel.isLocal(message.from))
+        if (message == null || waitingFor(message.from))
             super.response(message);
     }
 
@@ -55,4 +55,10 @@ public class DatacenterWriteResponseHandler extends WriteResponseHandler
         // or we may fail the consistency level guarantees (see #833, #8058)
         return consistencyLevel.blockFor(keyspace) + consistencyLevel.countLocalEndpoints(pendingEndpoints);
     }
+
+    @Override
+    protected boolean waitingFor(InetAddress from)
+    {
+        return consistencyLevel.isLocal(from);
+    }
 }
