@@ -1,6 +1,4 @@
-package org.apache.cassandra.service;
 /*
- *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -9,21 +7,20 @@ package org.apache.cassandra.service;
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing,
- * software distributed under the License is distributed on an
- * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- * KIND, either express or implied.  See the License for the
- * specific language governing permissions and limitations
- * under the License.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
-
+package org.apache.cassandra.service;
 
 import java.io.IOException;
 
-import org.apache.cassandra.thrift.CassandraDaemon;
+import org.apache.cassandra.config.DatabaseDescriptor;
+import org.apache.cassandra.service.CassandraDaemon;
 
 /**
  * An embedded, in-memory cassandra storage service that listens
@@ -31,7 +28,7 @@ import org.apache.cassandra.thrift.CassandraDaemon;
  * This kind of service is useful when running unit tests of
  * services using cassandra for example.
  *
- * See {@link EmbeddedCassandraServiceTest} for usage.
+ * See {@link org.apache.cassandra.service.EmbeddedCassandraServiceTest} for usage.
  * <p>
  * This is the implementation of https://issues.apache.org/jira/browse/CASSANDRA-740
  * <p>
@@ -53,6 +50,7 @@ public class EmbeddedCassandraService
     public void start() throws IOException
     {
         cassandraDaemon = new CassandraDaemon();
+        DatabaseDescriptor.setDaemonInitialized();
         cassandraDaemon.init(null);
         cassandraDaemon.start();
     }
