@@ -28,9 +28,9 @@ public class StatusBackup extends NodeToolCmd
     @Override
     public void execute(NodeProbe probe)
     {
-        System.out.println(
+        probe.output().out.println(
                 probe.isIncrementalBackupsEnabled()
                 ? "running"
                 : "not running");
     }
-}
\ No newline at end of file
+}
