@@ -17,20 +17,20 @@
  */
 package org.apache.cassandra.transport.messages;
 
-import com.google.common.annotations.VisibleForTesting;
+import java.util.List;
+
 import io.netty.buffer.ByteBuf;
 import io.netty.handler.codec.CodecException;
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Predicate;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.cassandra.cql3.functions.FunctionName;
 import org.apache.cassandra.db.ConsistencyLevel;
 import org.apache.cassandra.db.WriteType;
 import org.apache.cassandra.exceptions.*;
-import org.apache.cassandra.transport.CBUtil;
-import org.apache.cassandra.transport.Message;
-import org.apache.cassandra.transport.ProtocolException;
-import org.apache.cassandra.transport.ServerError;
+import org.apache.cassandra.transport.*;
 import org.apache.cassandra.utils.MD5Digest;
 
 /**
@@ -76,6 +76,25 @@ public class ErrorMessage extends Message.Response
                 case TRUNCATE_ERROR:
                     te = new TruncateException(msg);
                     break;
+                case WRITE_FAILURE: 
+                case READ_FAILURE:
+                    {
+                        ConsistencyLevel cl = CBUtil.readConsistencyLevel(body);
+                        int received = body.readInt();
+                        int blockFor = body.readInt();
+                        int failure = body.readInt();
+                        if (code == ExceptionCode.WRITE_FAILURE)
+                        {
+                            WriteType writeType = Enum.valueOf(WriteType.class, CBUtil.readString(body));
+                            te = new WriteFailureException(cl, received, failure, blockFor, writeType);
+                        }
+                        else
+                        {
+                            byte dataPresent = body.readByte();
+                            te = new ReadFailureException(cl, received, failure, blockFor, dataPresent != 0);   
+                        }
+                    }
+                    break;
                 case WRITE_TIMEOUT:
                 case READ_TIMEOUT:
                     ConsistencyLevel cl = CBUtil.readConsistencyLevel(body);
@@ -92,6 +111,12 @@ public class ErrorMessage extends Message.Response
                         te = new ReadTimeoutException(cl, received, blockFor, dataPresent != 0);
                     }
                     break;
+                case FUNCTION_FAILURE:
+                    String fKeyspace = CBUtil.readString(body);
+                    String fName = CBUtil.readString(body);
+                    List<String> argTypes = CBUtil.readStringList(body);
+                    te = new FunctionExecutionException(new FunctionName(fKeyspace, fName), argTypes, msg);
+                    break;
                 case UNPREPARED:
                     {
                         MD5Digest id = MD5Digest.wrap(CBUtil.readBytes(body));
@@ -124,21 +149,40 @@ public class ErrorMessage extends Message.Response
 
         public void encode(ErrorMessage msg, ByteBuf dest, int version)
         {
-            dest.writeInt(msg.error.code().value);
-            CBUtil.writeString(msg.error.getMessage(), dest);
+            final TransportException err = getBackwardsCompatibleException(msg, version);
+            dest.writeInt(err.code().value);
+            String errorString = err.getMessage() == null ? "" : err.getMessage();
+            CBUtil.writeString(errorString, dest);
 
-            switch (msg.error.code())
+            switch (err.code())
             {
                 case UNAVAILABLE:
-                    UnavailableException ue = (UnavailableException)msg.error;
+                    UnavailableException ue = (UnavailableException)err;
                     CBUtil.writeConsistencyLevel(ue.consistency, dest);
                     dest.writeInt(ue.required);
                     dest.writeInt(ue.alive);
                     break;
+                case WRITE_FAILURE:
+                case READ_FAILURE:
+                    {
+                        RequestFailureException rfe = (RequestFailureException)err;
+                        boolean isWrite = err.code() == ExceptionCode.WRITE_FAILURE;
+
+                        CBUtil.writeConsistencyLevel(rfe.consistency, dest);
+                        dest.writeInt(rfe.received);
+                        dest.writeInt(rfe.blockFor);
+                        dest.writeInt(rfe.failures);
+
+                        if (isWrite)
+                            CBUtil.writeString(((WriteFailureException)rfe).writeType.toString(), dest);
+                        else
+                            dest.writeByte((byte)(((ReadFailureException)rfe).dataPresent ? 1 : 0));
+                    }
+                    break;
                 case WRITE_TIMEOUT:
                 case READ_TIMEOUT:
-                    RequestTimeoutException rte = (RequestTimeoutException)msg.error;
-                    boolean isWrite = msg.error.code() == ExceptionCode.WRITE_TIMEOUT;
+                    RequestTimeoutException rte = (RequestTimeoutException)err;
+                    boolean isWrite = err.code() == ExceptionCode.WRITE_TIMEOUT;
 
                     CBUtil.writeConsistencyLevel(rte.consistency, dest);
                     dest.writeInt(rte.received);
@@ -148,12 +192,18 @@ public class ErrorMessage extends Message.Response
                     else
                         dest.writeByte((byte)(((ReadTimeoutException)rte).dataPresent ? 1 : 0));
                     break;
+                case FUNCTION_FAILURE:
+                    FunctionExecutionException fee = (FunctionExecutionException)msg.error;
+                    CBUtil.writeString(fee.functionName.keyspace, dest);
+                    CBUtil.writeString(fee.functionName.name, dest);
+                    CBUtil.writeStringList(fee.argTypes, dest);
+                    break;
                 case UNPREPARED:
-                    PreparedQueryNotFoundException pqnfe = (PreparedQueryNotFoundException)msg.error;
+                    PreparedQueryNotFoundException pqnfe = (PreparedQueryNotFoundException)err;
                     CBUtil.writeBytes(pqnfe.id.bytes, dest);
                     break;
                 case ALREADY_EXISTS:
-                    AlreadyExistsException aee = (AlreadyExistsException)msg.error;
+                    AlreadyExistsException aee = (AlreadyExistsException)err;
                     CBUtil.writeString(aee.ksName, dest);
                     CBUtil.writeString(aee.cfName, dest);
                     break;
@@ -162,26 +212,43 @@ public class ErrorMessage extends Message.Response
 
         public int encodedSize(ErrorMessage msg, int version)
         {
-            int size = 4 + CBUtil.sizeOfString(msg.error.getMessage());
-            switch (msg.error.code())
+            final TransportException err = getBackwardsCompatibleException(msg, version);
+            String errorString = err.getMessage() == null ? "" : err.getMessage();
+            int size = 4 + CBUtil.sizeOfString(errorString);
+            switch (err.code())
             {
                 case UNAVAILABLE:
-                    UnavailableException ue = (UnavailableException)msg.error;
+                    UnavailableException ue = (UnavailableException)err;
                     size += CBUtil.sizeOfConsistencyLevel(ue.consistency) + 8;
                     break;
+                case WRITE_FAILURE:
+                case READ_FAILURE:
+                    {
+                        RequestFailureException rfe = (RequestFailureException)err;
+                        boolean isWrite = err.code() == ExceptionCode.WRITE_FAILURE;
+                        size += CBUtil.sizeOfConsistencyLevel(rfe.consistency) + 4 + 4 + 4;
+                        size += isWrite ? CBUtil.sizeOfString(((WriteFailureException)rfe).writeType.toString()) : 1;
+                    }
+                    break;
                 case WRITE_TIMEOUT:
                 case READ_TIMEOUT:
-                    RequestTimeoutException rte = (RequestTimeoutException)msg.error;
-                    boolean isWrite = msg.error.code() == ExceptionCode.WRITE_TIMEOUT;
+                    RequestTimeoutException rte = (RequestTimeoutException)err;
+                    boolean isWrite = err.code() == ExceptionCode.WRITE_TIMEOUT;
                     size += CBUtil.sizeOfConsistencyLevel(rte.consistency) + 8;
                     size += isWrite ? CBUtil.sizeOfString(((WriteTimeoutException)rte).writeType.toString()) : 1;
                     break;
+                case FUNCTION_FAILURE:
+                    FunctionExecutionException fee = (FunctionExecutionException)msg.error;
+                    size += CBUtil.sizeOfString(fee.functionName.keyspace);
+                    size += CBUtil.sizeOfString(fee.functionName.name);
+                    size += CBUtil.sizeOfStringList(fee.argTypes);
+                    break;
                 case UNPREPARED:
-                    PreparedQueryNotFoundException pqnfe = (PreparedQueryNotFoundException)msg.error;
+                    PreparedQueryNotFoundException pqnfe = (PreparedQueryNotFoundException)err;
                     size += CBUtil.sizeOfBytes(pqnfe.id.bytes);
                     break;
                 case ALREADY_EXISTS:
-                    AlreadyExistsException aee = (AlreadyExistsException)msg.error;
+                    AlreadyExistsException aee = (AlreadyExistsException)err;
                     size += CBUtil.sizeOfString(aee.ksName);
                     size += CBUtil.sizeOfString(aee.cfName);
                     break;
@@ -190,6 +257,26 @@ public class ErrorMessage extends Message.Response
         }
     };
 
+    private static TransportException getBackwardsCompatibleException(ErrorMessage msg, int version)
+    {
+        if (version < Server.VERSION_4)
+        {
+            switch (msg.error.code())
+            {
+                case READ_FAILURE:
+                    ReadFailureException rfe = (ReadFailureException) msg.error;
+                    return new ReadTimeoutException(rfe.consistency, rfe.received, rfe.blockFor, rfe.dataPresent);
+                case WRITE_FAILURE:
+                    WriteFailureException wfe = (WriteFailureException) msg.error;
+                    return new WriteTimeoutException(wfe.writeType, wfe.consistency, wfe.received, wfe.blockFor);
+                case FUNCTION_FAILURE:
+                    return new InvalidRequestException(msg.toString());
+            }
+        }
+
+        return msg.error;
+    }
+
     // We need to figure error codes out (#3979)
     public final TransportException error;
 
@@ -244,7 +331,18 @@ public class ErrorMessage extends Message.Response
         }
 
         if (e instanceof TransportException)
-            return new ErrorMessage((TransportException)e, streamId);
+        {
+            ErrorMessage message = new ErrorMessage((TransportException) e, streamId);
+            if (e instanceof ProtocolException)
+            {
+                // if the driver attempted to connect with a protocol version lower than the minimum supported
+                // version, respond with a protocol error message with the correct frame header for that version
+                Integer attemptedLowProtocolVersion = ((ProtocolException) e).getAttemptedLowProtocolVersion();
+                if (attemptedLowProtocolVersion != null)
+                    message.forcedProtocolVersion = attemptedLowProtocolVersion;
+            }
+            return message;
+        }
 
         // Unexpected exception
         if (unexpectedExceptionHandler == null || !unexpectedExceptionHandler.apply(e))
