@@ -28,6 +28,7 @@ import java.net.InetAddress;
 import java.util.StringTokenizer;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.TimeoutException;
 
 import org.apache.cassandra.concurrent.DebuggableScheduledThreadPoolExecutor;
 import org.apache.cassandra.gms.ApplicationState;
@@ -143,6 +144,11 @@ public class BackgroundActivityMonitor
         return 0.0;
     }
 
+    public void shutdownAndWait(long timeout, TimeUnit unit) throws TimeoutException, InterruptedException
+    {
+        ExecutorUtils.shutdownAndWait(timeout, unit, reportThread);
+    }
+
     public class BackgroundActivityReporter implements Runnable
     {
         public void run()
