@@ -17,14 +17,13 @@
  */
 package org.apache.cassandra.utils;
 
-import java.nio.ByteBuffer;
-
 import com.google.common.annotations.VisibleForTesting;
 
+import org.apache.cassandra.utils.concurrent.Ref;
 import org.apache.cassandra.utils.concurrent.WrappedSharedCloseable;
 import org.apache.cassandra.utils.obs.IBitSet;
 
-public abstract class BloomFilter extends WrappedSharedCloseable implements IFilter
+public class BloomFilter extends WrappedSharedCloseable implements IFilter
 {
     private static final ThreadLocal<long[]> reusableIndexes = new ThreadLocal<long[]>()
     {
@@ -36,19 +35,31 @@ public abstract class BloomFilter extends WrappedSharedCloseable implements IFil
 
     public final IBitSet bitset;
     public final int hashCount;
+    /**
+     * CASSANDRA-8413: 3.0 (inverted) bloom filters have no 'static' bits caused by using the same upper bits
+     * for both bloom filter and token distribution.
+     */
+    public final boolean oldBfHashOrder;
 
-    BloomFilter(int hashCount, IBitSet bitset)
+    BloomFilter(int hashCount, IBitSet bitset, boolean oldBfHashOrder)
     {
         super(bitset);
         this.hashCount = hashCount;
         this.bitset = bitset;
+        this.oldBfHashOrder = oldBfHashOrder;
     }
 
-    BloomFilter(BloomFilter copy)
+    private BloomFilter(BloomFilter copy)
     {
         super(copy);
         this.hashCount = copy.hashCount;
         this.bitset = copy.bitset;
+        this.oldBfHashOrder = copy.oldBfHashOrder;
+    }
+
+    public long serializedSize()
+    {
+        return BloomFilterSerializer.serializedSize(this);
     }
 
     // Murmur is faster than an SHA-based approach and provides as-good collision
@@ -56,17 +67,16 @@ public abstract class BloomFilter extends WrappedSharedCloseable implements IFil
     // http://www.eecs.harvard.edu/~kirsch/pubs/bbbf/esa06.pdf
     // does prove to work in actual tests, and is obviously faster
     // than performing further iterations of murmur.
-    protected abstract void hash(ByteBuffer b, int position, int remaining, long seed, long[] result);
 
     // tests ask for ridiculous numbers of hashes so here is a special case for them
     // rather than using the threadLocal like we do in production
     @VisibleForTesting
-    public long[] getHashBuckets(ByteBuffer key, int hashCount, long max)
+    public long[] getHashBuckets(FilterKey key, int hashCount, long max)
     {
         long[] hash = new long[2];
-        hash(key, key.position(), key.remaining(), 0L, hash);
+        key.filterHash(hash);
         long[] indexes = new long[hashCount];
-        setIndexes(hash[0], hash[1], hashCount, max, indexes);
+        setIndexes(hash[1], hash[0], hashCount, max, indexes);
         return indexes;
     }
 
@@ -74,18 +84,25 @@ public abstract class BloomFilter extends WrappedSharedCloseable implements IFil
     // to avoid generating a lot of garbage since stack allocation currently does not support stores
     // (CASSANDRA-6609).  it returns the array so that the caller does not need to perform
     // a second threadlocal lookup.
-    private long[] indexes(ByteBuffer key)
+    private long[] indexes(FilterKey key)
     {
         // we use the same array both for storing the hash result, and for storing the indexes we return,
         // so that we do not need to allocate two arrays.
         long[] indexes = reusableIndexes.get();
-        hash(key, key.position(), key.remaining(), 0L, indexes);
-        setIndexes(indexes[0], indexes[1], hashCount, bitset.capacity(), indexes);
+        key.filterHash(indexes);
+        setIndexes(indexes[1], indexes[0], hashCount, bitset.capacity(), indexes);
         return indexes;
     }
 
     private void setIndexes(long base, long inc, int count, long max, long[] results)
     {
+        if (oldBfHashOrder)
+        {
+            long x = inc;
+            inc = base;
+            base = x;
+        }
+
         for (int i = 0; i < count; i++)
         {
             results[i] = FBUtilities.abs(base % max);
@@ -93,7 +110,7 @@ public abstract class BloomFilter extends WrappedSharedCloseable implements IFil
         }
     }
 
-    public void add(ByteBuffer key)
+    public void add(FilterKey key)
     {
         long[] indexes = indexes(key);
         for (int i = 0; i < hashCount; i++)
@@ -102,7 +119,7 @@ public abstract class BloomFilter extends WrappedSharedCloseable implements IFil
         }
     }
 
-    public final boolean isPresent(ByteBuffer key)
+    public final boolean isPresent(FilterKey key)
     {
         long[] indexes = indexes(key);
         for (int i = 0; i < hashCount; i++)
@@ -119,4 +136,26 @@ public abstract class BloomFilter extends WrappedSharedCloseable implements IFil
     {
         bitset.clear();
     }
+
+    public IFilter sharedCopy()
+    {
+        return new BloomFilter(this);
+    }
+
+    @Override
+    public long offHeapSize()
+    {
+        return bitset.offHeapSize();
+    }
+
+    public String toString()
+    {
+        return "BloomFilter[hashCount=" + hashCount + ";oldBfHashOrder=" + oldBfHashOrder + ";capacity=" + bitset.capacity() + ']';
+    }
+
+    public void addTo(Ref.IdentityCollection identities)
+    {
+        super.addTo(identities);
+        bitset.addTo(identities);
+    }
 }
