@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -15,55 +15,52 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.cassandra.utils;
 
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
 
 import com.google.common.base.Function;
+import com.google.common.util.concurrent.Uninterruptibles;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.cassandra.concurrent.DebuggableScheduledThreadPoolExecutor;
-import org.cliffc.high_scale_lib.NonBlockingHashMap;
 
 public class ExpiringMap<K, V>
 {
     private static final Logger logger = LoggerFactory.getLogger(ExpiringMap.class);
     private volatile boolean shutdown;
 
-    private static class CacheableObject<T>
+    public static class CacheableObject<T>
     {
-        private final T value;
+        public final T value;
+        public final long timeout;
         private final long createdAt;
-        private final long expiration;
 
-        CacheableObject(T o, long e)
+        private CacheableObject(T value, long timeout)
         {
-            assert o != null;
-            value = o;
-            expiration = e;
-            createdAt = System.currentTimeMillis();
+            assert value != null;
+            this.value = value;
+            this.timeout = timeout;
+            this.createdAt = Clock.instance.nanoTime();
         }
 
-        T getValue()
+        private boolean isReadyToDieAt(long atNano)
         {
-            return value;
-        }
-
-        boolean isReadyToDieAt(long time)
-        {
-            return ((time - createdAt) > expiration);
+            return atNano - createdAt > TimeUnit.MILLISECONDS.toNanos(timeout);
         }
     }
 
     // if we use more ExpiringMaps we may want to add multiple threads to this executor
     private static final ScheduledExecutorService service = new DebuggableScheduledThreadPoolExecutor("EXPIRING-MAP-REAPER");
 
-    private final NonBlockingHashMap<K, CacheableObject<V>> cache = new NonBlockingHashMap<K, CacheableObject<V>>();
+    private final ConcurrentMap<K, CacheableObject<V>> cache = new ConcurrentHashMap<K, CacheableObject<V>>();
     private final long defaultExpiration;
 
     public ExpiringMap(long defaultExpiration)
@@ -75,7 +72,7 @@ public class ExpiringMap<K, V>
      *
      * @param defaultExpiration the TTL for objects in the cache in milliseconds
      */
-    public ExpiringMap(long defaultExpiration, final Function<Pair<K,V>, ?> postExpireHook)
+    public ExpiringMap(long defaultExpiration, final Function<Pair<K,CacheableObject<V>>, ?> postExpireHook)
     {
         this.defaultExpiration = defaultExpiration;
 
@@ -88,16 +85,18 @@ public class ExpiringMap<K, V>
         {
             public void run()
             {
-                long start = System.currentTimeMillis();
+                long start = Clock.instance.nanoTime();
                 int n = 0;
                 for (Map.Entry<K, CacheableObject<V>> entry : cache.entrySet())
                 {
                     if (entry.getValue().isReadyToDieAt(start))
                     {
-                        cache.remove(entry.getKey());
-                        n++;
-                        if (postExpireHook != null)
-                            postExpireHook.apply(new Pair<K, V>(entry.getKey(), entry.getValue().getValue()));
+                        if (cache.remove(entry.getKey()) != null)
+                        {
+                            n++;
+                            if (postExpireHook != null)
+                                postExpireHook.apply(Pair.create(entry.getKey(), entry.getValue()));
+                        }
                     }
                 }
                 logger.trace("Expired {} entries", n);
@@ -106,12 +105,12 @@ public class ExpiringMap<K, V>
         service.scheduleWithFixedDelay(runnable, defaultExpiration / 2, defaultExpiration / 2, TimeUnit.MILLISECONDS);
     }
 
-    public void shutdownBlocking()
+    public boolean shutdownBlocking()
     {
         service.shutdown();
         try
         {
-            service.awaitTermination(defaultExpiration * 2, TimeUnit.MILLISECONDS);
+            return service.awaitTermination(defaultExpiration * 2, TimeUnit.MILLISECONDS);
         }
         catch (InterruptedException e)
         {
@@ -122,6 +121,11 @@ public class ExpiringMap<K, V>
     public void reset()
     {
         shutdown = false;
+        clear();
+    }
+
+    public void clear()
+    {
         cache.clear();
     }
 
@@ -138,31 +142,27 @@ public class ExpiringMap<K, V>
             // So we'll just sit on this thread until the rest of the server shutdown completes.
             //
             // See comments in CustomTThreadPoolServer.serve, CASSANDRA-3335, and CASSANDRA-3727.
-            try
-            {
-                Thread.sleep(Long.MAX_VALUE);
-            }
-            catch (InterruptedException e)
-            {
-                throw new AssertionError(e);
-            }
+            Uninterruptibles.sleepUninterruptibly(Long.MAX_VALUE, TimeUnit.NANOSECONDS);
         }
         CacheableObject<V> previous = cache.put(key, new CacheableObject<V>(value, timeout));
-        return (previous == null) ? null : previous.getValue();
+        return (previous == null) ? null : previous.value;
     }
 
     public V get(K key)
     {
         CacheableObject<V> co = cache.get(key);
-        return co == null ? null : co.getValue();
+        return co == null ? null : co.value;
     }
 
     public V remove(K key)
     {
         CacheableObject<V> co = cache.remove(key);
-        return co == null ? null : co.getValue();
+        return co == null ? null : co.value;
     }
 
+    /**
+     * @return System.nanoTime() when key was put into the map.
+     */
     public long getAge(K key)
     {
         CacheableObject<V> co = cache.get(key);
