@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -15,19 +15,16 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.cassandra.utils;
 
-import java.net.InetAddress;
-import java.net.UnknownHostException;
 import java.nio.ByteBuffer;
 import java.security.SecureRandom;
 import java.util.Random;
 
 public class GuidGenerator {
-    private static Random myRand;
-    private static SecureRandom mySecureRand;
-    private static String s_id;
+    private static final Random myRand;
+    private static final SecureRandom mySecureRand;
+    private static final String s_id;
 
     static {
         if (System.getProperty("java.security.egd") == null) {
@@ -37,9 +34,9 @@ public class GuidGenerator {
         long secureInitializer = mySecureRand.nextLong();
         myRand = new Random(secureInitializer);
         try {
-            s_id = InetAddress.getLocalHost().toString();
+            s_id = FBUtilities.getLocalAddress().toString();
         }
-        catch (UnknownHostException e) {
+        catch (RuntimeException e) {
             throw new AssertionError(e);
         }
     }
@@ -77,10 +74,10 @@ public class GuidGenerator {
         long rand = 0;
         rand = myRand.nextLong();
         sbValueBeforeMD5.append(s_id)
-        				.append(":")
-        				.append(Long.toString(time))
-        				.append(":")
-        				.append(Long.toString(rand));
+                        .append(":")
+                        .append(Long.toString(time))
+                        .append(":")
+                        .append(Long.toString(rand));
 
         String valueBeforeMD5 = sbValueBeforeMD5.toString();
         return ByteBuffer.wrap(FBUtilities.threadLocalMD5Digest().digest(valueBeforeMD5.getBytes()));
