@@ -1,94 +1,125 @@
 /*
-* Licensed to the Apache Software Foundation (ASF) under one
-* or more contributor license agreements.  See the NOTICE file
-* distributed with this work for additional information
-* regarding copyright ownership.  The ASF licenses this file
-* to you under the Apache License, Version 2.0 (the
-* "License"); you may not use this file except in compliance
-* with the License.  You may obtain a copy of the License at
-*
-*    http://www.apache.org/licenses/LICENSE-2.0
-*
-* Unless required by applicable law or agreed to in writing,
-* software distributed under the License is distributed on an
-* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-* KIND, either express or implied.  See the License for the
-* specific language governing permissions and limitations
-* under the License.
-*/
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package org.apache.cassandra.io.sstable;
 
-import java.util.*;
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.List;
 
 import com.google.common.collect.Iterables;
+import com.google.common.util.concurrent.RateLimiter;
+
+import org.junit.BeforeClass;
 import org.junit.Test;
 
 import org.apache.cassandra.SchemaLoader;
 import org.apache.cassandra.Util;
-import org.apache.cassandra.db.*;
-import org.apache.cassandra.db.columniterator.IdentityQueryFilter;
-import org.apache.cassandra.dht.*;
+import org.apache.cassandra.config.CFMetaData;
+import org.apache.cassandra.db.ColumnFamilyStore;
+import org.apache.cassandra.db.DataRange;
+import org.apache.cassandra.db.DecoratedKey;
+import org.apache.cassandra.db.Keyspace;
+import org.apache.cassandra.db.PartitionPosition;
+import org.apache.cassandra.db.RowUpdateBuilder;
+import org.apache.cassandra.db.Slices;
+import org.apache.cassandra.db.filter.ClusteringIndexSliceFilter;
+import org.apache.cassandra.db.filter.ColumnFilter;
+import org.apache.cassandra.dht.AbstractBounds;
+import org.apache.cassandra.dht.ByteOrderedPartitioner;
+import org.apache.cassandra.dht.Range;
+import org.apache.cassandra.dht.Token;
+import org.apache.cassandra.io.sstable.format.SSTableReader;
+import org.apache.cassandra.io.sstable.format.SSTableReadsListener;
+import org.apache.cassandra.schema.KeyspaceParams;
 import org.apache.cassandra.utils.ByteBufferUtil;
 
 import static org.apache.cassandra.dht.AbstractBounds.isEmpty;
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertTrue;
 
-public class SSTableScannerTest extends SchemaLoader
+public class SSTableScannerTest
 {
-    public static final String KEYSPACE = "Keyspace1";
+    public static final String KEYSPACE = "SSTableScannerTest";
     public static final String TABLE = "Standard1";
 
+    @BeforeClass
+    public static void defineSchema() throws Exception
+    {
+        SchemaLoader.prepareServer();
+        SchemaLoader.createKeyspace(KEYSPACE,
+                                    KeyspaceParams.simple(1),
+                                    SchemaLoader.standardCFMD(KEYSPACE, TABLE));
+    }
+
     private static String toKey(int key)
     {
         return String.format("%03d", key);
     }
 
     // we produce all DataRange variations that produce an inclusive start and exclusive end range
-    private static Iterable<DataRange> dataRanges(int start, int end)
+    private static Iterable<DataRange> dataRanges(CFMetaData metadata, int start, int end)
     {
         if (end < start)
-            return dataRanges(start, end, false, true);
-        return Iterables.concat(dataRanges(start, end, false, false),
-                                dataRanges(start, end, false, true),
-                                dataRanges(start, end, true, false),
-                                dataRanges(start, end, true, true)
+            return dataRanges(metadata, start, end, false, true);
+        return Iterables.concat(dataRanges(metadata, start, end, false, false),
+                                dataRanges(metadata, start, end, false, true),
+                                dataRanges(metadata, start, end, true, false),
+                                dataRanges(metadata, start, end, true, true)
         );
     }
 
-    private static Iterable<DataRange> dataRanges(int start, int end, boolean inclusiveStart, boolean inclusiveEnd)
+    private static Iterable<DataRange> dataRanges(CFMetaData metadata, int start, int end, boolean inclusiveStart, boolean inclusiveEnd)
     {
         List<DataRange> ranges = new ArrayList<>();
         if (start == end + 1)
         {
             assert !inclusiveStart && inclusiveEnd;
-            ranges.add(dataRange(min(start), false, max(end), true));
-            ranges.add(dataRange(min(start), false, min(end + 1), true));
-            ranges.add(dataRange(max(start - 1), false, max(end), true));
-            ranges.add(dataRange(dk(start - 1), false, dk(start - 1), true));
+            ranges.add(dataRange(metadata, min(start), false, max(end), true));
+            ranges.add(dataRange(metadata, min(start), false, min(end + 1), true));
+            ranges.add(dataRange(metadata, max(start - 1), false, max(end), true));
+            ranges.add(dataRange(metadata, dk(start - 1), false, dk(start - 1), true));
         }
         else
         {
-            for (RowPosition s : starts(start, inclusiveStart))
+            for (PartitionPosition s : starts(start, inclusiveStart))
             {
-                for (RowPosition e : ends(end, inclusiveEnd))
+                for (PartitionPosition e : ends(end, inclusiveEnd))
                 {
                     if (end < start && e.compareTo(s) > 0)
                         continue;
                     if (!isEmpty(new AbstractBounds.Boundary<>(s, inclusiveStart), new AbstractBounds.Boundary<>(e, inclusiveEnd)))
                         continue;
-                    ranges.add(dataRange(s, inclusiveStart, e, inclusiveEnd));
+                    ranges.add(dataRange(metadata, s, inclusiveStart, e, inclusiveEnd));
                 }
             }
         }
         return ranges;
     }
 
-    private static Iterable<RowPosition> starts(int key, boolean inclusive)
+    private static Iterable<PartitionPosition> starts(int key, boolean inclusive)
     {
         return Arrays.asList(min(key), max(key - 1), dk(inclusive ? key : key - 1));
     }
 
-    private static Iterable<RowPosition> ends(int key, boolean inclusive)
+    private static Iterable<PartitionPosition> ends(int key, boolean inclusive)
     {
         return Arrays.asList(max(key), min(key + 1), dk(inclusive ? key : key + 1));
     }
@@ -100,28 +131,31 @@ public class SSTableScannerTest extends SchemaLoader
 
     private static Token token(int key)
     {
-        return key == Integer.MIN_VALUE ? ByteOrderedPartitioner.MINIMUM : new BytesToken(toKey(key).getBytes());
+        return key == Integer.MIN_VALUE ? ByteOrderedPartitioner.MINIMUM : new ByteOrderedPartitioner.BytesToken(toKey(key).getBytes());
     }
 
-    private static RowPosition min(int key)
+    private static PartitionPosition min(int key)
     {
         return token(key).minKeyBound();
     }
 
-    private static RowPosition max(int key)
+    private static PartitionPosition max(int key)
     {
         return token(key).maxKeyBound();
     }
 
-    private static DataRange dataRange(RowPosition start, boolean startInclusive, RowPosition end, boolean endInclusive)
+    private static DataRange dataRange(CFMetaData metadata, PartitionPosition start, boolean startInclusive, PartitionPosition end, boolean endInclusive)
     {
-        return new DataRange(AbstractBounds.bounds(start, startInclusive, end, endInclusive), new IdentityQueryFilter());
+        Slices.Builder sb = new Slices.Builder(metadata.comparator);
+        ClusteringIndexSliceFilter filter = new ClusteringIndexSliceFilter(sb.build(), false);
+
+        return new DataRange(AbstractBounds.bounds(start, startInclusive, end, endInclusive), filter);
     }
 
     private static Range<Token> rangeFor(int start, int end)
     {
-        return new Range<Token>(new BytesToken(toKey(start).getBytes()),
-                                end == Integer.MIN_VALUE ? ByteOrderedPartitioner.MINIMUM : new BytesToken(toKey(end).getBytes()));
+        return new Range<Token>(new ByteOrderedPartitioner.BytesToken(toKey(start).getBytes()),
+                                end == Integer.MIN_VALUE ? ByteOrderedPartitioner.MINIMUM : new ByteOrderedPartitioner.BytesToken(toKey(end).getBytes()));
     }
 
     private static Collection<Range<Token>> makeRanges(int ... keys)
@@ -132,25 +166,37 @@ public class SSTableScannerTest extends SchemaLoader
         return ranges;
     }
 
-    private static void insertRowWithKey(int key)
+    private static void insertRowWithKey(CFMetaData metadata, int key)
     {
         long timestamp = System.currentTimeMillis();
-        DecoratedKey decoratedKey = Util.dk(toKey(key));
-        Mutation rm = new Mutation(KEYSPACE, decoratedKey.getKey());
-        rm.add(TABLE, Util.cellname("col"), ByteBufferUtil.EMPTY_BYTE_BUFFER, timestamp, 1000);
-        rm.apply();
+
+        new RowUpdateBuilder(metadata, timestamp, toKey(key))
+            .clustering("col")
+            .add("val", ByteBufferUtil.EMPTY_BYTE_BUFFER)
+            .build()
+            .applyUnsafe();
+
     }
 
     private static void assertScanMatches(SSTableReader sstable, int scanStart, int scanEnd, int ... boundaries)
     {
         assert boundaries.length % 2 == 0;
-        for (DataRange range : dataRanges(scanStart, scanEnd))
+        for (DataRange range : dataRanges(sstable.metadata, scanStart, scanEnd))
         {
-            ISSTableScanner scanner = sstable.getScanner(range);
-            for (int b = 0 ; b < boundaries.length ; b += 2)
-                for (int i = boundaries[b] ; i <= boundaries[b + 1] ; i++)
-                    assertEquals(toKey(i), new String(scanner.next().getKey().getKey().array()));
-            assertFalse(scanner.hasNext());
+            try(ISSTableScanner scanner = sstable.getScanner(ColumnFilter.all(sstable.metadata),
+                                                             range,
+                                                             false,
+                                                             SSTableReadsListener.NOOP_LISTENER))
+            {
+                for (int b = 0; b < boundaries.length; b += 2)
+                    for (int i = boundaries[b]; i <= boundaries[b + 1]; i++)
+                        assertEquals(toKey(i), new String(scanner.next().partitionKey().getKey().array()));
+                assertFalse(scanner.hasNext());
+            }
+            catch (Exception e)
+            {
+                throw new RuntimeException(e);
+            }
         }
     }
 
@@ -160,7 +206,7 @@ public class SSTableScannerTest extends SchemaLoader
     }
 
     @Test
-    public void testSingleDataRange()
+    public void testSingleDataRange() throws IOException
     {
         Keyspace keyspace = Keyspace.open(KEYSPACE);
         ColumnFamilyStore store = keyspace.getColumnFamilyStore(TABLE);
@@ -170,16 +216,18 @@ public class SSTableScannerTest extends SchemaLoader
         store.disableAutoCompaction();
 
         for (int i = 2; i < 10; i++)
-            insertRowWithKey(i);
+            insertRowWithKey(store.metadata, i);
         store.forceBlockingFlush();
 
-        assertEquals(1, store.getSSTables().size());
-        SSTableReader sstable = store.getSSTables().iterator().next();
+        assertEquals(1, store.getLiveSSTables().size());
+        SSTableReader sstable = store.getLiveSSTables().iterator().next();
 
         // full range scan
-        ISSTableScanner scanner = sstable.getScanner();
+        ISSTableScanner scanner = sstable.getScanner(RateLimiter.create(Double.MAX_VALUE));
         for (int i = 2; i < 10; i++)
-            assertEquals(toKey(i), new String(scanner.next().getKey().getKey().array()));
+            assertEquals(toKey(i), new String(scanner.next().partitionKey().getKey().array()));
+
+        scanner.close();
 
         // a simple read of a chunk in the middle
         assertScanMatches(sstable, 3, 6, 3, 6);
@@ -243,7 +291,7 @@ public class SSTableScannerTest extends SchemaLoader
         assertScanMatches(sstable, 1, 0, 2, 9);
     }
 
-    private static void assertScanContainsRanges(ISSTableScanner scanner, int ... rangePairs)
+    private static void assertScanContainsRanges(ISSTableScanner scanner, int ... rangePairs) throws IOException
     {
         assert rangePairs.length % 2 == 0;
 
@@ -255,14 +303,15 @@ public class SSTableScannerTest extends SchemaLoader
             for (int expected = rangeStart; expected <= rangeEnd; expected++)
             {
                 assertTrue(String.format("Expected to see key %03d", expected), scanner.hasNext());
-                assertEquals(toKey(expected), new String(scanner.next().getKey().getKey().array()));
+                assertEquals(toKey(expected), new String(scanner.next().partitionKey().getKey().array()));
             }
         }
         assertFalse(scanner.hasNext());
+        scanner.close();
     }
 
     @Test
-    public void testMultipleRanges()
+    public void testMultipleRanges() throws IOException
     {
         Keyspace keyspace = Keyspace.open(KEYSPACE);
         ColumnFamilyStore store = keyspace.getColumnFamilyStore(TABLE);
@@ -273,14 +322,14 @@ public class SSTableScannerTest extends SchemaLoader
 
         for (int i = 0; i < 3; i++)
             for (int j = 2; j < 10; j++)
-                insertRowWithKey(i * 100 + j);
+                insertRowWithKey(store.metadata, i * 100 + j);
         store.forceBlockingFlush();
 
-        assertEquals(1, store.getSSTables().size());
-        SSTableReader sstable = store.getSSTables().iterator().next();
+        assertEquals(1, store.getLiveSSTables().size());
+        SSTableReader sstable = store.getLiveSSTables().iterator().next();
 
         // full range scan
-        ISSTableScanner fullScanner = sstable.getScanner();
+        ISSTableScanner fullScanner = sstable.getScanner(RateLimiter.create(Double.MAX_VALUE));
         assertScanContainsRanges(fullScanner,
                                  2, 9,
                                  102, 109,
@@ -394,7 +443,7 @@ public class SSTableScannerTest extends SchemaLoader
     }
 
     @Test
-    public void testSingleKeyMultipleRanges()
+    public void testSingleKeyMultipleRanges() throws IOException
     {
         Keyspace keyspace = Keyspace.open(KEYSPACE);
         ColumnFamilyStore store = keyspace.getColumnFamilyStore(TABLE);
@@ -403,14 +452,14 @@ public class SSTableScannerTest extends SchemaLoader
         // disable compaction while flushing
         store.disableAutoCompaction();
 
-        insertRowWithKey(205);
+        insertRowWithKey(store.metadata, 205);
         store.forceBlockingFlush();
 
-        assertEquals(1, store.getSSTables().size());
-        SSTableReader sstable = store.getSSTables().iterator().next();
+        assertEquals(1, store.getLiveSSTables().size());
+        SSTableReader sstable = store.getLiveSSTables().iterator().next();
 
         // full range scan
-        ISSTableScanner fullScanner = sstable.getScanner();
+        ISSTableScanner fullScanner = sstable.getScanner(RateLimiter.create(Double.MAX_VALUE));
         assertScanContainsRanges(fullScanner, 205, 205);
 
         // scan three ranges separately
