@@ -17,12 +17,14 @@
  */
 package org.apache.cassandra.triggers;
 
-import java.net.InetAddress;
 import java.nio.ByteBuffer;
 import java.util.Collection;
 import java.util.Collections;
 
-import org.junit.*;
+import org.junit.AfterClass;
+import org.junit.Before;
+import org.junit.BeforeClass;
+import org.junit.Test;
 
 import org.apache.cassandra.SchemaLoader;
 import org.apache.cassandra.config.Schema;
@@ -31,10 +33,13 @@ import org.apache.cassandra.cql3.UntypedResultSet;
 import org.apache.cassandra.db.*;
 import org.apache.cassandra.db.ConsistencyLevel;
 import org.apache.cassandra.db.Mutation;
+import org.apache.cassandra.db.partitions.Partition;
+import org.apache.cassandra.db.partitions.PartitionUpdate;
 import org.apache.cassandra.exceptions.ConfigurationException;
 import org.apache.cassandra.exceptions.RequestExecutionException;
 import org.apache.cassandra.service.StorageService;
 import org.apache.cassandra.thrift.*;
+import org.apache.cassandra.utils.FBUtilities;
 import org.apache.thrift.protocol.TBinaryProtocol;
 
 import static org.apache.cassandra.utils.ByteBufferUtil.bytes;
@@ -55,15 +60,15 @@ public class TriggersTest
     public static void beforeTest() throws ConfigurationException
     {
         SchemaLoader.loadSchema();
-        StorageService.instance.initServer(0);
     }
 
     @Before
     public void setup() throws Exception
     {
+        StorageService.instance.initServer(0);
         if (thriftServer == null || ! thriftServer.isRunning())
         {
-            thriftServer = new ThriftServer(InetAddress.getLocalHost(), 9170, 50);
+            thriftServer = new ThriftServer(FBUtilities.getLocalAddress(), 9170, 50);
             thriftServer.start();
         }
 
@@ -102,7 +107,7 @@ public class TriggersTest
     {
         String cql = String.format("INSERT INTO %s.%s (k, v1) VALUES (0, 0)", ksName, cfName);
         QueryProcessor.process(cql, ConsistencyLevel.ONE);
-        assertUpdateIsAugmented(0);
+        assertUpdateIsAugmented(0, "v1", 0);
     }
 
     @Test
@@ -113,7 +118,7 @@ public class TriggersTest
                                    "APPLY BATCH",
                                    ksName, cfName);
         QueryProcessor.process(cql, ConsistencyLevel.ONE);
-        assertUpdateIsAugmented(1);
+        assertUpdateIsAugmented(1, "v1", 1);
     }
 
     @Test
@@ -122,14 +127,14 @@ public class TriggersTest
         Cassandra.Client client = new Cassandra.Client(
                                         new TBinaryProtocol(
                                             new TFramedTransportFactory().openTransport(
-                                                InetAddress.getLocalHost().getHostName(), 9170)));
+                                                FBUtilities.getLocalAddress().getHostName(), 9170)));
         client.set_keyspace(ksName);
         client.insert(bytes(2),
                       new ColumnParent(cfName),
                       getColumnForInsert("v1", 2),
                       org.apache.cassandra.thrift.ConsistencyLevel.ONE);
 
-        assertUpdateIsAugmented(2);
+        assertUpdateIsAugmented(2, "v1", 2);
     }
 
     @Test
@@ -138,7 +143,7 @@ public class TriggersTest
         Cassandra.Client client = new Cassandra.Client(
                                     new TBinaryProtocol(
                                         new TFramedTransportFactory().openTransport(
-                                            InetAddress.getLocalHost().getHostName(), 9170)));
+                                            FBUtilities.getLocalAddress().getHostName(), 9170)));
         client.set_keyspace(ksName);
         org.apache.cassandra.thrift.Mutation mutation = new org.apache.cassandra.thrift.Mutation();
         ColumnOrSuperColumn cosc = new ColumnOrSuperColumn();
@@ -150,7 +155,7 @@ public class TriggersTest
                                                               Collections.singletonList(mutation))),
             org.apache.cassandra.thrift.ConsistencyLevel.ONE);
 
-        assertUpdateIsAugmented(3);
+        assertUpdateIsAugmented(3, "v1", 3);
     }
 
     @Test
@@ -158,7 +163,7 @@ public class TriggersTest
     {
         String cql = String.format("INSERT INTO %s.%s (k, v1) VALUES (4, 4) IF NOT EXISTS", ksName, cfName);
         QueryProcessor.process(cql, ConsistencyLevel.ONE);
-        assertUpdateIsAugmented(4);
+        assertUpdateIsAugmented(4, "v1", 4);
     }
 
     @Test
@@ -170,7 +175,7 @@ public class TriggersTest
                                    "APPLY BATCH",
                                     ksName, cfName);
         QueryProcessor.process(cql, ConsistencyLevel.ONE);
-        assertUpdateIsAugmented(5);
+        assertUpdateIsAugmented(5, "v1", 5);
     }
 
     @Test
@@ -179,7 +184,7 @@ public class TriggersTest
         Cassandra.Client client = new Cassandra.Client(
                 new TBinaryProtocol(
                         new TFramedTransportFactory().openTransport(
-                                InetAddress.getLocalHost().getHostName(), 9170)));
+                                FBUtilities.getLocalAddress().getHostName(), 9170)));
         client.set_keyspace(ksName);
         client.cas(bytes(6),
                    cfName,
@@ -188,7 +193,7 @@ public class TriggersTest
                    org.apache.cassandra.thrift.ConsistencyLevel.LOCAL_SERIAL,
                    org.apache.cassandra.thrift.ConsistencyLevel.ONE);
 
-        assertUpdateIsAugmented(6);
+        assertUpdateIsAugmented(6, "v1", 6);
     }
 
     // Unfortunately, an IRE thrown from StorageProxy.cas
@@ -237,7 +242,7 @@ public class TriggersTest
             Cassandra.Client client = new Cassandra.Client(
                     new TBinaryProtocol(
                             new TFramedTransportFactory().openTransport(
-                                    InetAddress.getLocalHost().getHostName(), 9170)));
+                                    FBUtilities.getLocalAddress().getHostName(), 9170)));
             client.set_keyspace(ksName);
             client.cas(bytes(9),
                        cf,
@@ -262,7 +267,7 @@ public class TriggersTest
             Cassandra.Client client = new Cassandra.Client(
                     new TBinaryProtocol(
                             new TFramedTransportFactory().openTransport(
-                                    InetAddress.getLocalHost().getHostName(), 9170)));
+                                    FBUtilities.getLocalAddress().getHostName(), 9170)));
             client.set_keyspace(ksName);
             client.cas(bytes(10),
                        cf,
@@ -277,29 +282,6 @@ public class TriggersTest
         }
     }
 
-    @Test(expected=RuntimeException.class)
-    public void ifTriggerThrowsErrorNoMutationsAreApplied() throws Exception
-    {
-        String cf = "cf" + System.nanoTime();
-        try
-        {
-            setupTableWithTrigger(cf, ErrorTrigger.class);
-            String cql = String.format("INSERT INTO %s.%s (k, v1) VALUES (11, 11)", ksName, cf);
-            QueryProcessor.process(cql, ConsistencyLevel.ONE);
-        }
-        catch (Exception e)
-        {
-            Throwable cause = e.getCause();
-            assertTrue((cause instanceof org.apache.cassandra.exceptions.InvalidRequestException));
-            assertTrue(cause.getMessage().equals(ErrorTrigger.MESSAGE));
-            throw e;
-        }
-        finally
-        {
-            assertUpdateNotExecuted(cf, 11);
-        }
-    }
-
     private void setupTableWithTrigger(String cf, Class<? extends ITrigger> triggerImpl)
     throws RequestExecutionException
     {
@@ -312,12 +294,18 @@ public class TriggersTest
         QueryProcessor.process(cql, ConsistencyLevel.ONE);
     }
 
-    private void assertUpdateIsAugmented(int key)
+    private void assertUpdateIsAugmented(int key, String originColumnName, Object originColumnValue)
     {
-        UntypedResultSet rs = QueryProcessor.executeInternal(
-                                String.format("SELECT * FROM %s.%s WHERE k=%s", ksName, cfName, key));
-        assertTrue(String.format("Expected value (%s) for augmented cell v2 was not found", key), rs.one().has("v2"));
-        assertEquals(999, rs.one().getInt("v2"));
+        UntypedResultSet rs = QueryProcessor.process(String.format("SELECT * FROM %s.%s WHERE k=%s", ksName, cfName, key), ConsistencyLevel.ONE);
+        assertRowValue(rs.one(), key, "v2", 999); // from trigger
+        assertRowValue(rs.one(), key, originColumnName, originColumnValue); // from original update
+    }
+    
+    private void assertRowValue(UntypedResultSet.Row row, int key, String columnName, Object columnValue)
+    {
+        assertTrue(String.format("Expected value (%s) for augmented cell %s was not found", key, columnName),
+                   row.has(columnName));
+        assertEquals(columnValue, row.getInt(columnName));
     }
 
     private void assertUpdateNotExecuted(String cf, int key)
@@ -330,7 +318,7 @@ public class TriggersTest
     private org.apache.cassandra.thrift.Column getColumnForInsert(String columnName, int value)
     {
         org.apache.cassandra.thrift.Column column = new org.apache.cassandra.thrift.Column();
-        column.setName(Schema.instance.getCFMetaData(ksName, cfName).comparator.asAbstractType().fromString(columnName));
+        column.setName(LegacyLayout.makeLegacyComparator(Schema.instance.getCFMetaData(ksName, cfName)).fromString(columnName));
         column.setValue(bytes(value));
         column.setTimestamp(System.currentTimeMillis());
         return column;
@@ -338,43 +326,35 @@ public class TriggersTest
 
     public static class TestTrigger implements ITrigger
     {
-        public Collection<Mutation> augment(ByteBuffer key, ColumnFamily update)
+        public Collection<Mutation> augment(Partition partition)
         {
-            ColumnFamily extraUpdate = update.cloneMeShallow(ArrayBackedSortedColumns.factory, false);
-            extraUpdate.addColumn(new BufferCell(update.metadata().comparator.makeCellName(bytes("v2")), bytes(999)));
-            return Collections.singletonList(new Mutation(ksName, key, extraUpdate));
+            RowUpdateBuilder update = new RowUpdateBuilder(partition.metadata(), FBUtilities.timestampMicros(), partition.partitionKey().getKey());
+            update.add("v2", 999);
+
+            return Collections.singletonList(update.build());
         }
     }
 
     public static class CrossPartitionTrigger implements ITrigger
     {
-        public Collection<Mutation> augment(ByteBuffer key, ColumnFamily update)
+        public Collection<Mutation> augment(Partition partition)
         {
-            ColumnFamily extraUpdate = update.cloneMeShallow(ArrayBackedSortedColumns.factory, false);
-            extraUpdate.addColumn(new BufferCell(update.metadata().comparator.makeCellName(bytes("v2")), bytes(999)));
+            RowUpdateBuilder update = new RowUpdateBuilder(partition.metadata(), FBUtilities.timestampMicros(), toInt(partition.partitionKey().getKey()) + 1000);
+            update.add("v2", 999);
 
-            int newKey = toInt(key) + 1000;
-            return Collections.singletonList(new Mutation(ksName, bytes(newKey), extraUpdate));
+            return Collections.singletonList(update.build());
         }
     }
 
     public static class CrossTableTrigger implements ITrigger
     {
-        public Collection<Mutation> augment(ByteBuffer key, ColumnFamily update)
+        public Collection<Mutation> augment(Partition partition)
         {
-            ColumnFamily extraUpdate = ArrayBackedSortedColumns.factory.create(ksName, otherCf);
-            extraUpdate.addColumn(new BufferCell(extraUpdate.metadata().comparator.makeCellName(bytes("v2")), bytes(999)));
-            return Collections.singletonList(new Mutation(ksName, key, extraUpdate));
+
+            RowUpdateBuilder update = new RowUpdateBuilder(Schema.instance.getCFMetaData(ksName, otherCf), FBUtilities.timestampMicros(), partition.partitionKey().getKey());
+            update.add("v2", 999);
+
+            return Collections.singletonList(update.build());
         }
     }
-
-    public static class ErrorTrigger implements ITrigger
-    {
-        public static final String MESSAGE = "Thrown by ErrorTrigger";
-        public Collection<Mutation> augment(ByteBuffer partitionKey, ColumnFamily update)
-        {
-            throw new org.apache.cassandra.exceptions.InvalidRequestException(MESSAGE);
-        }
-    }
-
 }
