@@ -182,12 +182,11 @@ public class StressSettings implements Serializable
         {
             synchronized (this)
             {
-                String currentNode = node.randomNode();
                 if (client != null)
                     return client;
 
                 EncryptionOptions.ClientEncryptionOptions encOptions = transport.getEncryptionOptions();
-                JavaDriverClient c = new JavaDriverClient(this, currentNode, port.nativePort, encOptions);
+                JavaDriverClient c = new JavaDriverClient(this, node.nodes, port.nativePort, encOptions);
                 c.connect(mode.compression());
                 if (setKeyspace)
                     c.execute("USE \"" + schema.keyspace + "\";", org.apache.cassandra.db.ConsistencyLevel.ONE);
